@@ -1,4 +1,4 @@
-/*
+/* vi:set et sw=2 sts=2 cin cino=t0,f0,(0,{s,>2s,n-s,^-s,e-s:
  * Copyright © 2017 Red Hat, Inc
  *
  * This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 
 #include <glib/gi18n.h>
 
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 
 #include "flatpak-builtins.h"
 #include "flatpak-builtins-utils.h"
@@ -149,17 +149,11 @@ flatpak_builtin_remote_info (int argc, char **argv, GCancellable *cancellable, G
     {
       if (opt_commit)
         commit = g_strdup (opt_commit);
-      else
+      else if (!flatpak_remote_state_lookup_ref (state, flatpak_decomposed_get_ref (ref),
+                                                 &commit, NULL, NULL, NULL, error))
         {
-          flatpak_remote_state_lookup_ref (state, flatpak_decomposed_get_ref (ref), &commit, NULL, NULL, NULL, error);
-          if (commit == NULL)
-            {
-              if (error != NULL && *error == NULL)
-                flatpak_fail_error (error, FLATPAK_ERROR_REF_NOT_FOUND,
-                                    _("Couldn't find latest checksum for ref %s in remote %s"),
-                                    flatpak_decomposed_get_ref (ref), remote);
-              return FALSE;
-            }
+          g_assert (error == NULL || *error != NULL);
+          return FALSE;
         }
     }
   else
@@ -185,29 +179,25 @@ flatpak_builtin_remote_info (int argc, char **argv, GCancellable *cancellable, G
       int len;
       int rows, cols;
       int width;
-      g_autoptr(AsStore) store = as_store_new ();
-      AsApp *app = NULL;
+      g_autoptr(AsMetadata) mdata = as_metadata_new ();
+      AsComponent *app = NULL;
       const char *version = NULL;
       const char *license = NULL;
       g_autofree char *id = flatpak_decomposed_dup_id (ref);
 
       flatpak_get_window_size (&rows, &cols);
 
-#if AS_CHECK_VERSION (0, 6, 1)
-      as_store_set_add_flags (store, as_store_get_add_flags (store) | AS_STORE_ADD_FLAG_USE_UNIQUE_ID);
-#endif
-
-      flatpak_dir_load_appstream_store (preferred_dir, remote, id, store, NULL, NULL);
-      app = as_store_find_app (store, flatpak_decomposed_get_ref (ref));
+      flatpak_dir_load_appstream_store (preferred_dir, remote, id, mdata, NULL, NULL);
+      app = as_store_find_app (mdata, flatpak_decomposed_get_ref (ref));
       if (app)
         {
-          const char *name = as_app_get_localized_name (app);
-          const char *comment = as_app_get_localized_comment (app);
+          const char *name = as_component_get_name (app);
+          const char *comment = as_component_get_summary (app);
 
           print_wrapped (MIN (cols, 80), "\n%s - %s\n", name, comment);
 
           version = as_app_get_version (app);
-          license = as_app_get_project_license (app);
+          license = as_component_get_project_license (app);
         }
 
       if (commit_v)
@@ -441,7 +431,10 @@ flatpak_builtin_remote_info (int argc, char **argv, GCancellable *cancellable, G
 
           if (opt_show_metadata)
             {
-              g_print ("%s", xa_metadata ? xa_metadata : "");
+              if (xa_metadata != NULL)
+                flatpak_print_escaped_string (xa_metadata,
+                                              FLATPAK_ESCAPE_ALLOW_NEWLINES
+                                              | FLATPAK_ESCAPE_DO_NOT_QUOTE);
               if (xa_metadata == NULL || !g_str_has_suffix (xa_metadata, "\n"))
                 g_print ("\n");
             }
