@@ -734,6 +734,7 @@ test_full (void)
   g_autofree gchar *create_dir = g_build_filename (subdir, "create-dir", NULL);
   g_autofree gchar *create_dir2 = g_build_filename (subdir, "create-dir2", NULL);
   gsize i;
+  gboolean ok;
 
   glnx_shutil_rm_rf_at (-1, subdir, NULL, &error);
 
@@ -789,30 +790,55 @@ test_full (void)
                                        FLATPAK_FILESYSTEM_MODE_READ_WRITE);
   flatpak_exports_add_host_os_expose (exports,
                                       FLATPAK_FILESYSTEM_MODE_READ_ONLY);
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_WRITE,
-                                   expose_rw);
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   expose_ro);
-  flatpak_exports_add_path_tmpfs (exports, hide_below_expose);
-  flatpak_exports_add_path_expose_or_hide (exports,
-                                           FLATPAK_FILESYSTEM_MODE_NONE,
-                                           hide);
-  flatpak_exports_add_path_expose_or_hide (exports,
-                                           FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                           dont_hide);
-  flatpak_exports_add_path_expose_or_hide (exports,
-                                           FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                           enoent);
-  flatpak_exports_add_path_expose_or_hide (exports,
-                                           FLATPAK_FILESYSTEM_MODE_READ_WRITE,
-                                           rel_link);
-  flatpak_exports_add_path_expose_or_hide (exports,
-                                           FLATPAK_FILESYSTEM_MODE_READ_WRITE,
-                                           abs_link);
-  flatpak_exports_add_path_dir (exports, create_dir);
-  flatpak_exports_add_path_dir (exports, create_dir2);
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_WRITE,
+                                        expose_rw, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        expose_ro, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+  ok = flatpak_exports_add_path_tmpfs (exports, hide_below_expose, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+  ok = flatpak_exports_add_path_expose_or_hide (exports,
+                                                FLATPAK_FILESYSTEM_MODE_NONE,
+                                                hide, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+  ok = flatpak_exports_add_path_expose_or_hide (exports,
+                                                FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                                dont_hide, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+
+  ok = flatpak_exports_add_path_expose_or_hide (exports,
+                                                FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                                enoent, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
+  ok = flatpak_exports_add_path_expose_or_hide (exports,
+                                                FLATPAK_FILESYSTEM_MODE_READ_WRITE,
+                                                rel_link, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+  ok = flatpak_exports_add_path_expose_or_hide (exports,
+                                                FLATPAK_FILESYSTEM_MODE_READ_WRITE,
+                                                abs_link, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+  ok = flatpak_exports_add_path_dir (exports, create_dir, &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+
+  ok = flatpak_exports_add_path_dir (exports, create_dir2, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND);
+  g_assert_false (ok);
+  g_clear_error (&error);
 
   g_assert_cmpuint (flatpak_exports_path_get_mode (exports, expose_rw), ==,
                     FLATPAK_FILESYSTEM_MODE_READ_WRITE);
@@ -1233,6 +1259,39 @@ test_exports_debian_merged (void)
   g_assert_cmpuint (i, ==, bwrap->argv->len);
 }
 
+static const struct
+{
+  const char *tried;
+  const char *because;
+}
+reserved_filesystems[] =
+{
+  { "/", "/.flatpak-info" },
+  { "/.flatpak-info", "/.flatpak-info" },
+  { "/app", "/app" },
+  { "/app/foo", "/app" },
+  { "/bin", "/bin" },
+  { "/bin/sh", "/bin" },
+  { "/dev", "/dev" },
+  { "/etc", "/etc" },
+  { "/etc/passwd", "/etc" },
+  { "/lib", "/lib" },
+  { "/lib/ld-linux.so.2", "/lib" },
+  { "/lib64", "/lib64" },
+  { "/lib64/ld-linux-x86-64.so.2", "/lib64" },
+  { "/proc", "/proc" },
+  { "/proc/1", "/proc" },
+  { "/proc/sys/net", "/proc" },
+  { "/run", "/run/flatpak" },
+  { "/run/flatpak/foo/bar", "/run/flatpak" },
+  { "/run/host/foo", "/run/host" },
+  { "/sbin", "/sbin" },
+  { "/sbin/ldconfig", "/sbin" },
+  { "/usr", "/usr" },
+  { "/usr/bin/env", "/usr" },
+  { "/usr/foo/bar", "/usr" },
+};
+
 static void
 test_exports_ignored (void)
 {
@@ -1240,62 +1299,31 @@ test_exports_ignored (void)
   g_autoptr(FlatpakExports) exports = flatpak_exports_new ();
   gsize i;
 
-  /* These paths are chosen so that they probably exist, with the
-   * exception of /app */
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/app");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/etc");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/etc/passwd");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/usr");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/usr/bin/env");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/dev");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/dev/full");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/proc");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/proc/1");
+  for (i = 0; i < G_N_ELEMENTS (reserved_filesystems); i++)
+    {
+      const char *tried = reserved_filesystems[i].tried;
+      const char *because = reserved_filesystems[i].because;
+      g_autoptr(GError) error = NULL;
+      gboolean ok;
 
-  /* These probably exist, and are merged into /usr on systems with
-   * the /usr merge */
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/bin");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/bin/sh");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/lib");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/lib/ld-linux.so.2");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/lib64");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/lib64/ld-linux-x86-64.so.2");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/sbin");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/sbin/ldconfig");
+      ok = flatpak_exports_add_path_expose (exports,
+                                            FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                            tried,
+                                            &error);
+      g_assert_nonnull (error);
+      g_assert_nonnull (error->message);
+      g_test_message ("Trying to export %s -> %s", tried, error->message);
+      g_assert_false (ok);
+
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTABLE_FILE))
+        {
+          g_autofree char *pattern = g_strdup_printf ("Path \"%s\" is reserved by Flatpak",
+                                                      because);
+
+          g_test_message ("Expecting to see pattern: %s", pattern);
+          g_assert_nonnull (strstr (error->message, pattern));
+        }
+    }
 
   flatpak_bwrap_add_arg (bwrap, "bwrap");
   flatpak_exports_append_bwrap_args (exports, bwrap);
@@ -1332,6 +1360,7 @@ test_exports_unusual (void)
     { "home", FAKE_SYMLINK, "var/home" },
     { "lib", FAKE_SYMLINK, "usr/lib" },
     { "recursion", FAKE_SYMLINK, "recursion" },
+    { "symlink-to-root", FAKE_SYMLINK, "." },
     { "tmp", FAKE_SYMLINK, "TMP" },
     { "usr/bin", FAKE_DIR },
     { "usr/lib", FAKE_DIR },
@@ -1344,35 +1373,79 @@ test_exports_unusual (void)
   g_autoptr(FlatpakBwrap) bwrap = flatpak_bwrap_new (NULL);
   g_autoptr(FlatpakExports) exports = NULL;
   gsize i;
+  g_autoptr(GError) error = NULL;
+  gboolean ok;
 
   exports = test_host_exports_setup (files,
                                      FLATPAK_FILESYSTEM_MODE_NONE,
                                      FLATPAK_FILESYSTEM_MODE_READ_ONLY);
   flatpak_exports_set_test_flags (exports, FLATPAK_EXPORTS_TEST_FLAGS_AUTOFS);
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/broken-autofs");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/dangling-link");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/home/me");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/nonexistent");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/recursion");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "/tmp");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_WRITE,
-                                   "/var/tmp");
-  flatpak_exports_add_path_expose (exports,
-                                   FLATPAK_FILESYSTEM_MODE_READ_ONLY,
-                                   "not-absolute");
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/broken-autofs", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK);
+  g_test_message ("attempting to export /broken-autofs: %s", error->message);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/dangling-link", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND);
+  g_test_message ("attempting to export /dangling-link: %s", error->message);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/home/me", &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/nonexistent", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND);
+  g_test_message ("attempting to export /nonexistent: %s", error->message);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/recursion", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_TOO_MANY_LINKS);
+  g_test_message ("attempting to export /recursion: %s", error->message);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/symlink-to-root", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTABLE_FILE);
+  g_test_message ("attempting to export /symlink-to-root: %s", error->message);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "/tmp", &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_WRITE,
+                                        "/var/tmp", &error);
+  g_assert_no_error (error);
+  g_assert_true (ok);
+
+  ok = flatpak_exports_add_path_expose (exports,
+                                        FLATPAK_FILESYSTEM_MODE_READ_ONLY,
+                                        "not-absolute", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME);
+  g_test_message ("attempting to export not-absolute: %s", error->message);
+  g_assert_false (ok);
+  g_clear_error (&error);
+
   test_host_exports_finish (exports, bwrap);
 
   i = 0;
@@ -1401,6 +1474,9 @@ main (int argc, char *argv[])
 {
   int res;
 
+  /* Do not call setlocale() here: some tests look at untranslated error
+   * messages. */
+
   isolated_test_dir_global_setup ();
 
   g_test_init (&argc, &argv, NULL);
